package internal

import (
	"github.com/open-io/oio-exporter/common"
	"runtime"
	"time"
)

type daemon struct {
	memAllocMax uint64
	interval    time.Duration
}

func MakeDaemon(interval time.Duration) *daemon {
	return &daemon{
		memAllocMax: 0,
		interval:    interval,
	}
}

func (d *daemon) Run(m *common.MetricData) {
	for {
		var mem runtime.MemStats
		runtime.ReadMemStats(&mem)
		if mem.Alloc > d.memAllocMax {
			d.memAllocMax = mem.Alloc
		}
		m.Set(&common.Metric{
			Name:   "self_mem_alloc_current",
			Labels: [][]string{},
			Value:  float64(mem.Alloc),
		})
		m.Set(&common.Metric{
			Name:   "self_mem_alloc_max",
			Labels: [][]string{},
			Value:  float64(d.memAllocMax),
		})
		m.Set(&common.Metric{
			Name:   "self_num_goroutines",
			Labels: [][]string{},
			Value:  float64(runtime.NumGoroutine()),
		})
		time.Sleep(d.interval)
	}
}
