package main

import (
	"io/ioutil"
	"log"
	"net/http"
	"net/http/httptest"
	"strings"
	"testing"

	"github.com/open-io/oio-exporter/common"
)

const HOSTNAME = "testnode"

func TestOIOExporter(t *testing.T) {
	exp := &oioExporter{
		host:      HOSTNAME,
		inventory: "./tmp/inventory.yml",
		hcTargets: []string{"localhost:12345"},
		metrics:   common.MakeMetricData(),
	}

	server := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		switch r.URL.Path {
		case "/metrics":
			exp.export(w, r)
		case "/config":
			exp.config(w, r)
		case "/reload":
			exp.reload(w, r)
		}
	}))
	defer server.Close()

	url := server.URL

	for _, route := range []string{"/metrics", "/config", "/reload"} {
		_, err := http.Get(url + route)
		if err != nil {
			log.Fatalln(err)
		}
	}

	exp.metrics.Set(&common.Metric{
		Name: "test_measurement",
		Labels: [][]string{
			[]string{"label1", "label1value"},
			[]string{"label2", "label2value"},
		},
		Value: float64(0.1),
	})

	resp, err := http.Get(url + "/metrics")
	if err != nil {
		log.Fatalln(err)
	}
	data, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		log.Fatalln(err)
	}

	lines := strings.Split(string(data), "\n")

	// Check that all measures contain the hostname
	for _, line := range lines[:len(lines)-1] {
		if !strings.Contains(line, HOSTNAME) {
			log.Fatalln("Measurement", line, "must contain the host label", HOSTNAME)
		}
	}
}
