package oiofs

import (
	"encoding/json"
	"fmt"
	"github.com/open-io/oio-exporter/common"
	"log"
	"net/http"
	"strings"
	"time"
)

type daemon struct {
	services **map[string][]*common.Service
	interval time.Duration
}

func MakeDaemon(services **map[string][]*common.Service, interval time.Duration) *daemon {
	return &daemon{
		services: services,
		interval: interval,
	}
}

func (d *daemon) Run(m *common.MetricData) {
	for {
		for serviceType, services := range **d.services {
			if serviceType != "oiofs" {
				continue
			}
			for _, svc := range services {
				target := fmt.Sprintf(svc.IP+":%d", svc.Port)
				res, err := http.Get("http://" + target + "/stats")
				if err != nil {
					m.IncrError("oiofs", serviceType)
					log.Println("WARN: oiofs stats collection failed", svc, err)
					continue
				}
				var data map[string]float64
				err = json.NewDecoder(res.Body).Decode(&data)
				if err != nil {
					m.IncrError("oiofs", serviceType)
					log.Println("WARN: oiofs stats collection failed", svc, err)
					continue
				}

				for k, v := range data {
					k = strings.ToLower(k)
					k = strings.Replace(k, "init_ctx", "initCtx", 1)
					parts := strings.Split(k, "_")

					metric := ""
					switch parts[len(parts)-1] {
					case "us":
						metric = "operation_time"
					case "count":
						metric = "operation_count"
					case "byte":
						metric = "bytes"
					case "microseconds":
						metric = "chunk_age"
					case "hit", "miss":
						metric = "cache_read"
					}
					m.Set(&common.Metric{
						Name: "oiofs_" + metric,
						Labels: [][]string{
							[]string{"service_type", serviceType},
							[]string{"service", target},
							[]string{"mountpoint", svc.Volume},
							[]string{"operation", strings.Trim(parts[1], " ")},
							[]string{"dimension", parts[2]},
							[]string{"module", parts[0]},
						},
						Value: v,
					})
				}
			}
		}
		time.Sleep(d.interval)
	}
}
